/*
 *  linux/fs/u2fs/file.c
 *
 *  Copyright (C) 1994 Yossi Gottlieb (yogo@math.tau.ac.il)
 *
 *  Based on code and information:
 *	BSD code,  Copyright (c) 1982, 1986 Regents of the University of California.
 *		   All rights reserved.
 *	linux/fs/minix	Copyright (C) 1991, 1992, 1993  Linus Torvalds
 *
 *  1996-04-19  Fixed a bug in ufs_file_read() by complete removal.
 *              Eric Delaunay <delaunay@lix.polytechnique.fr>
 *              Bruno Haible <haible@ma2s2.mathematik.uni-karlsruhe.de>
 *
 */

#include <linux/fs.h>

#include <linux/u2fs_fs.h>
#include "u2fs.h"

/*
 * Computes the disk block location corresponding to file position
 * block << inode->i_sb->s_blocksize_bits.
 */
static int u2fs_bmap(struct inode *inode, int block)
{
	struct u2fs_sb_info *sb = &inode->i_sb->u.u2fs_sb;

	/* Note that we have  s->s_blocksize == sb->fs_fsize, so we can
	 * pass `block' to u2fs_blknum() without conversion. */
	daddr_t blk = u2fs_blknum(inode, block);
	if (blk == -1)
		return 0; /* block not present, e.g. hole or past EOF */
	return sb->fs_block_base + fsbtodb(sb, blk);
}

static struct file_operations u2fs_file_operations = {
	NULL,			/* lseek - default */
	generic_file_read,	/* read */
	NULL,			/* write - none */
	NULL,			/* readdir - none */
	NULL,			/* select - none */
	NULL,			/* ioctl - none */
	generic_file_mmap,	/* mmap */
	NULL,			/* open - none */
	NULL,			/* release - none */
	file_fsync,		/* fsync */
	NULL,			/* fasync - none */
	NULL,			/* check_media_change */
	NULL			/* revalidate */
};

struct inode_operations u2fs_file_inode_operations = {
	&u2fs_file_operations,
	NULL,			/* create - none */
	NULL,			/* lookup - none */
	NULL,			/* link - none */
	NULL,			/* unlink - none */
	NULL,			/* symlink - none */
	NULL,			/* mkdir - none */
	NULL,			/* rmdir - none */
	NULL,			/* mknod - none */
	NULL,			/* rename - none */
	NULL,			/* readlink - none */
	NULL,			/* follow_link - none */
	generic_readpage,	/* readpage */
	NULL,			/* writepage - none */
	u2fs_bmap,		/* bmap - wrapper for u2fs_blknum */
	NULL,			/* truncate - none */
	NULL,			/* permission - none */
	NULL			/* smap - none */
};
